#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

import json
from datetime import datetime, timezone

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Any, Eq
from pyramid.decorator import reify
from zope.interface import Interface, implementer
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema import Bool, Choice, TextLine
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_catalog.query import CatalogResultSet
from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.shared.common.interfaces import CONTENT_MANAGER_CONTRIBUTORS, IBaseSharedTool, ISharedToolRoles, \
    IWfSharedContent, IWfSharedContentRoles, SHARED_CONTENT_TYPES_VOCABULARY
from pyams_content.shared.common.zmi.dashboard import DashboardTable
from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.interfaces import HIDDEN_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.interfaces import IPagelet, PageletCreatedEvent
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.schema import PrincipalField
from pyams_security.utility import get_principal
from pyams_security_views.zmi.interfaces import IObjectSecurityMenu
from pyams_sequence.interfaces import ISequentialIdInfo, ISequentialIntIds
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager, IHeaderViewletManager
from pyams_skin.schema.button import ResetButton, SubmitButton
from pyams_skin.viewlet.help import AlertMessage
from pyams_table.column import CheckBoxColumn
from pyams_table.interfaces import IColumn, IValues
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_viewlet.viewlet import EmptyViewlet, viewlet_config
from pyams_workflow.interfaces import IWorkflow, IWorkflowState, IWorkflowVersions
from pyams_workflow.versions import WorkflowHistoryItem, get_last_version
from pyams_zmi.form import AdminAddForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.search import SearchForm, SearchResultsView, SearchView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


def set_owner(context, request, new_owner, keep_owner_as_contributor=False):
    """Set content owner"""
    updated = False
    workflow = IWorkflow(context)
    translate = request.localizer.translate
    for version in IWorkflowVersions(context).get_versions():
        state = IWorkflowState(version)
        if state.state in workflow.readonly_states:
            continue
        roles = IWfSharedContentRoles(version, None)
        if roles is not None:
            [previous_owner] = roles.owner
            if previous_owner != new_owner:
                roles.owner = {new_owner}
                contributors = roles.contributors.copy()  # don't modify contributors in place!!
                if keep_owner_as_contributor:
                    contributors |= {previous_owner}
                else:
                    if previous_owner in contributors:
                        contributors.remove(previous_owner)
                roles.contributors = contributors
                state.history.append(
                    WorkflowHistoryItem(date=tztime(datetime.now(timezone.utc)),
                                        source_state=state.state,
                                        target_state=state.state,
                                        transition_id=MISSING_INFO,
                                        principal=request.principal.id,
                                        comment=translate(_("Owner changed: {} -> {}")).format(
                                            get_principal(request, previous_owner).title,
                                            get_principal(request, new_owner).title
                                        )))
                request.registry.notify(ObjectModifiedEvent(version))
                updated = True
    return updated


#
# Bulk owner change
#

@viewlet_config(name='change-owner.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IObjectSecurityMenu, weight=300,
                permission=MANAGE_SITE_PERMISSION)
class SharedToolOwnerChangeMenu(NavigationMenuItem):
    """Shared tool owner change menu"""

    label = _("Change owner")
    href = '#change-owner.html'


class ISharedToolOwnerChangeSearchInfo(Interface):
    """Shared tool owner change search form fields"""

    old_owner = Choice(title=_("Current owner"),
                       description=_("This is the name of the current owner"),
                       vocabulary=CONTENT_MANAGER_CONTRIBUTORS,
                       required=False)


class SharedToolOwnerChangeSearchForm(SearchForm):
    """Shared tool owner change search form"""

    title = _("Contents owner change form")

    ajax_form_handler = 'change-owner-search-results.html'
    _edit_permission = MANAGE_SITE_PERMISSION

    fields = Fields(ISharedToolOwnerChangeSearchInfo)


@pagelet_config(name='change-owner.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=MANAGE_SITE_PERMISSION)
class SharedToolOwnerChangeView(SearchView):
    """Shared tool owner change view"""

    title = _("Contents owner search form")
    header_label = _("Contents search form")
    search_form = SharedToolOwnerChangeSearchForm


class SharedToolOwnerChangeSearchResultsTable(DashboardTable):
    """Shared tool owner search results table"""

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        modules = attributes.get('table', {}).get('data-ams-modules', '')
        attributes.setdefault('table', {}).update({
            'data-ams-modules': f'{modules} container',
            'data-buttons': json.dumps(['copy', 'csv', 'print'])
        })
        return attributes


@adapter_config(name='checked',
                required=(IBaseSharedTool, IAdminLayer, SharedToolOwnerChangeSearchResultsTable),
                provides=IColumn)
class SharedToolOwnerChangeCheckedColumn(CheckBoxColumn):
    """Shared tool owner change checked column"""

    weight = 1
    sortable = 'false'

    def get_item_key(self, item):
        return 'owner_form.widgets.selection-input'

    def get_item_value(self, item):
        return str(ISequentialIdInfo(item).oid)

    def is_selected(self, item):
        return False

    def render_head_cell(self):
        return (f'<input type="checkbox" '
                f'       data-ams-change-handler="MyAMS.container.selectAllElements" />')

    def render_cell(self, item):
        return super().render_cell(item).replace(' />',
                                                 ' data-ams-click-handler="MyAMS.container.selectElement" '
                                                 ' data-ams-stop-propagation="true" />')


@adapter_config(required=(IBaseSharedTool, IPyAMSLayer, SharedToolOwnerChangeSearchResultsTable),
                provides=IValues)
class SharedToolOwnerChangeSearchResultsValues(ContextRequestViewAdapter):
    """Shared tool owner search results table values"""

    def get_params(self, data):
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(self.context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()))
        owner = data.get('old_owner')
        if owner:
            params &= Eq(catalog['role:owner'], owner)
        return params

    @property
    def values(self):
        """Search results values getter"""
        form = SharedToolOwnerChangeSearchForm(self.context, self.request)
        form.update()
        data, _errors = form.extract_data()
        params = self.get_params(data)
        catalog = get_utility(ICatalog)
        yield from unique_iter(
            map(get_last_version,
                CatalogResultSet(CatalogQuery(catalog).query(
                    params, sort_index='modified_date', reverse=True))))


class ISharedToolOwnerChangeFormFields(Interface):
    """Shared tool owner change form fields interface"""

    selection = TextLine(title=_("Selected items"),
                         required=False)

    new_owner = PrincipalField(title=_("New owner"),
                               description=_("Selected contents ownership will be assigned to "
                                             "selected principal"),
                               required=True)

    set_as_tool_contributor = Bool(title=_("Set as tool contributor"),
                                   description=_("If 'yes', selected owner will be granted the "
                                                 "contributor role on shared tool; otherwise, he "
                                                 "will get this role only on selected contents"),
                                   required=False,
                                   default=False)

    keep_owner_as_contributor = Bool(title=_("Keep previous owner as contributor"),
                                     description=_("If 'yes', the previous owner will still be "
                                                   "able to modify selected contents"),
                                     required=False,
                                     default=False)


class ISharedToolOwnerChangeFormActions(Interface):
    """Shared tool owner change form actions interface"""

    assign = SubmitButton(name='assign',
                          title=_("Change owner"))


@ajax_form_config(name='change-contents-owner.html',
                  context=IBaseSharedTool, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_PERMISSION)
@implementer(IObjectData)
class SharedToolOwnerChangeForm(AdminAddForm):
    """Shared tool owner change form"""

    prefix = 'owner_form.'
    title = None
    legend = _("New owner")
    hide_section = True

    fields = Fields(ISharedToolOwnerChangeFormFields)
    buttons = Buttons(ISharedToolOwnerChangeFormActions)

    @property
    def object_data(self):
        return {
            'ams-form-init-data-callback': 'MyAMS.container.getSelectedElements',
            'ams-container-source': 'owner_form.widgets.selection-input',
            'ams-container-target': 'owner_form.widgets.selection'
        }

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        selection = self.widgets.get('selection')
        if selection is not None:
            selection.mode = HIDDEN_MODE

    @handler(buttons['assign'])
    def handle_assign(self, action):
        super().handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        translate = self.request.localizer.translate
        selection = data.get('selection')
        if not selection:
            self.status = translate(_("Can't change owner for empty selection!"))
            return
        selection = list(map(int, selection.split(',')))
        new_owner = data.get('new_owner')
        # check tool contributor
        if data.get('set_as_tool_contributor'):
            roles = ISharedToolRoles(self.context)
            contributors = roles.contributors.copy()
            if new_owner not in contributors:
                contributors |= {new_owner}
                roles.contributors = contributors
        # set content owner
        updated = 0
        sequential_ids = get_utility(ISequentialIntIds)
        for oid in selection:
            content = sequential_ids.query_object_from_oid(oid)
            if content is not None:
                if set_owner(content, self.request, new_owner,
                             data.get('keep_owner_as_contributor')):
                    updated += 1
        return {
            'selected': len(selection),
            'updated': updated
        }


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolOwnerChangeForm),
                provides=IAJAXFormRenderer)
class SharedToolOwnerChangeFormRenderer(AJAXFormRenderer):
    """Shared tool owner change form renderer"""

    def render(self, changes):
        translate = self.request.localizer.translate
        if not changes:
            return {
                'status': 'info',
                'message': translate(_("Empty selection, no content owner was changed."))
            }
        selected = changes.get('selected', 0)
        changed = changes.get('updated', 0)
        if not changed:
            return {
                'status': 'info',
                'message': translate(_("{} contents selected, no owner changed !")).format(selected)
            }
        return {
            'status': 'success',
            'message': translate(_("{} contents selected, owner changed successfully for {} "
                                   "contents")).format(selected, changed),
            'callbacks': [{
                'callback': "MyAMS.container.removeSelectedElements",
                'options': {
                    'source': 'owner_form.widgets.selection-input'
                }
            }]
        }


@pagelet_config(name='change-owner-search-results.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=MANAGE_SITE_PERMISSION, xhr=True)
@template_config(template='templates/owner-change.pt',
                 layer=IPyAMSLayer)
class SharedToolOwnerChangeSearchResultsView(SearchResultsView):
    """Shared tool owner change search results view"""

    table_label = _("Search results")
    table_class = SharedToolOwnerChangeSearchResultsTable

    owner_change_form = None

    def __init__(self, context, request):
        super().__init__(context, request)
        request.registry.notify(PageletCreatedEvent(self))

    def update(self):
        super().update()
        if len(self.table.values) > 0:
            form = self.request.registry.queryMultiAdapter((self.context, self.request), IPagelet,
                                                           name='change-contents-owner.html')
            if form is not None:
                form.update()
                self.owner_change_form = form


@viewlet_config(name='pyams.content_header',
                layer=IAdminLayer, view=SharedToolOwnerChangeSearchResultsView,
                manager=IHeaderViewletManager, weight=10)
class SharedToolOwnerChangeSearchResultsViewHeaderViewlet(EmptyViewlet):
    """Shared tool owner change search results view header viewlet"""

    def render(self):
        return '<h1 class="mt-3"></h1>'


#
# Shared content owner change
#

@viewlet_config(name='change-owner.menu',
                context=IWfSharedContent, layer=IAdminLayer,
                manager=IObjectSecurityMenu, weight=10,
                permission=MANAGE_SITE_PERMISSION)
class WfSharedContentOwnerChangeMenu(NavigationMenuItem):
    """Shared content owner change menu"""

    label = _("Change owner")
    href = '#change-owner.html'


class IWfSharedContentOwnerChangeInfo(Interface):
    """Shared content owner change form fields"""

    new_owner = PrincipalField(title=_("New owner"),
                               description=_("The selected user will become the new content's owner"))

    keep_owner_as_contributor = Bool(title=_("Keep previous owner as contributor"),
                                     description=_("If 'yes', the previous owner will still be "
                                                   "able to modify this content"),
                                     required=False,
                                     default=False)


class IWfSharedContentOwnerChangeButtons(Interface):
    """Shared content owner change form buttons"""

    change = SubmitButton(name='change', title=_("Change owner"))
    reset = ResetButton(name='reset', title=_("Cancel"))


@ajax_form_config(name='change-owner.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_PERMISSION)
class WfSharedContentOwnerChangeForm(AdminAddForm):
    """Shared content owner change form"""

    title = _("Change content owner")
    legend = _("New owner selection")

    fields = Fields(IWfSharedContentOwnerChangeInfo)
    buttons = Buttons(IWfSharedContentOwnerChangeButtons)

    @handler(buttons['change'])
    def handle_change(self, action):
        super().handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        new_owner = data.get('new_owner')
        set_owner(self.context, self.request, new_owner,
                  data.get('keep_owner_as_contributor'))


@viewlet_config(name='help',
                context=IWfSharedContent, layer=IAdminLayer,
                view=WfSharedContentOwnerChangeForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentOwnerChangeFormHelp(AlertMessage):
    """Shared content owner change form help"""

    status = 'info'

    _message = _("All versions of this content which are not archived will be transferred to "
                 "newly selected owner")
    message_renderer = 'markdown'


@adapter_config(required=(IWfSharedContent, IAdminLayer, WfSharedContentOwnerChangeForm),
                provides=IAJAXFormRenderer)
class SharedContentOwnerChangeFormAJAXRenderer(ContextRequestViewAdapter):
    """Shared content owner change form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        return {
            'status': 'reload'
        }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.types module

"""

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.container.ordered import OrderedContainer
from zope.interface import implementer
from zope.lifecycleevent import IObjectAddedEvent
from zope.location.interfaces import ISublocations
from zope.schema import getFieldsInOrder
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from zope.traversing.interfaces import ITraversable

from pyams_content.component.extfile.interfaces import IExtFileContainerTarget
from pyams_content.component.links.interfaces import ILinkContainerTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content.component.thesaurus import ITagsInfo, ITagsTarget, IThemesInfo, IThemesTarget
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.reference.pictogram import IPictogramTable
from pyams_content.shared.common.interfaces import ISharedTool
from pyams_content.shared.common.interfaces.types import ALL_DATA_TYPES_VOCABULARY, \
    DATA_MANAGER_ANNOTATION_KEY, DATA_TYPES_VOCABULARY, DATA_TYPE_FIELDS_VOCABULARY, IDataType, \
    ITypedDataManager, ITypedSharedTool, IWfTypedSharedContent, VISIBLE_DATA_TYPES_VOCABULARY
from pyams_i18n.interfaces import II18n
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_sequence.reference import get_reference_target
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utilities_for, query_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(IDataType)
@implementer(IParagraphContainerTarget, ILinkContainerTarget, IExtFileContainerTarget,
             ITagsTarget, IThemesTarget)
class DataType(Persistent, Contained):
    """Base data type"""

    visible = FieldProperty(IDataType['visible'])
    label = FieldProperty(IDataType['label'])
    source_folder = FieldProperty(IDataType['source_folder'])
    navigation_label = FieldProperty(IDataType['navigation_label'])
    facets_label = FieldProperty(IDataType['facets_label'])
    facets_type_label = FieldProperty(IDataType['facets_type_label'])
    dashboard_label = FieldProperty(IDataType['dashboard_label'])
    color = FieldProperty(IDataType['color'])
    pictogram = FieldProperty(IDataType['pictogram'])
    pictogram_on = FieldProperty(IDataType['pictogram_on'])
    pictogram_off = FieldProperty(IDataType['pictogram_off'])
    display_as_tag = FieldProperty(IDataType['display_as_tag'])
    field_names = FieldProperty(IDataType['field_names'])

    def get_source_folder(self):
        """Source folder getter"""
        if self.source_folder is not None:
            return get_reference_target(self.source_folder)
        return None

    def get_pictogram(self):
        """Pictogram getter"""
        table = query_utility(IPictogramTable)
        if table is not None:
            return table.get(self.pictogram)
        return None


@adapter_config(required=IDataType,
                provides=ITypedSharedTool)
def datatype_shared_tool(context):
    """Datatype shared tool getter"""
    return get_parent(context, ITypedSharedTool)


@factory_config(ITypedDataManager)
class TypedDataManager(OrderedContainer):
    """Data types container persistent class"""

    def get_visible_items(self):
        """Iterator on visible data types"""
        yield from filter(lambda x: x.visible, self.values())


@adapter_config(required=IDataType,
                provides=IViewContextPermissionChecker)
class DatatypePermissionChecker(ContextAdapter):
    """Data type permission checker"""

    edit_permission = MANAGE_TOOL_PERMISSION


@implementer(ITypedSharedTool)
class TypedSharedToolMixin:
    """Typed shared tool"""

    shared_content_info_factory = None


@adapter_config(required=ITypedSharedTool,
                provides=ITypedDataManager)
def typed_shared_tool_data_manager_factory(context):
    """Types shared tool data manager factory"""
    return get_annotation_adapter(context, DATA_MANAGER_ANNOTATION_KEY, ITypedDataManager,
                                  name='++types++')


@adapter_config(name='types',
                required=ITypedSharedTool,
                provides=ITraversable)
class TypedSharedToolTypesNamespace(ContextAdapter):
    """Typed shared tool ++types++ namespace"""

    def traverse(self, name, furtherpath=None):
        """Namespace traverser"""
        return ITypedDataManager(self.context)


@adapter_config(name='types',
                required=ITypedSharedTool,
                provides=ISublocations)
class TypedSharedToolSublocations(ContextAdapter):
    """Typed shared tool sub-locations adapter"""

    def sublocations(self):
        """Sub-locations iterator"""
        yield from ITypedDataManager(self.context).values()


#
# Typed shared content
#

@implementer(IWfTypedSharedContent)
class WfTypedSharedContentMixin:
    """Typed shared content"""

    data_type = FieldProperty(IWfTypedSharedContent['data_type'])

    def get_data_type(self):
        """Datatype getter"""
        if not self.data_type:
            return None
        tool = ITypedSharedTool(self, None)
        if tool is not None:
            manager = ITypedDataManager(tool)
            return manager.get(self.data_type)
        return None

    @property
    def field_names(self):
        """Field names getter"""
        data_type = self.get_data_type()
        if data_type is not None:
            return data_type.field_names
        return None


@adapter_config(required=IWfTypedSharedContent,
                provides=ITypedSharedTool)
def typed_shared_content_shared_tool(context):
    """Typed shared content tool getter"""
    return get_parent(context, ITypedSharedTool)


@subscriber(IObjectAddedEvent, context_selector=IWfTypedSharedContent)
def handle_added_typed_shared_content(event):
    """Automatically assign tags and themes for newly created typed contents"""
    content = event.object
    data_type = content.get_data_type()
    if data_type is not None:
        if ITagsTarget.providedBy(data_type) and ITagsTarget.providedBy(content):
            tags_info = ITagsInfo(content)
            if not tags_info.tags:
                tags_info.tags = ITagsInfo(data_type).tags
        if IThemesTarget.providedBy(data_type) and IThemesTarget.providedBy(content):
            themes_info = IThemesInfo(content)
            if not themes_info.themes:  # don't remove previous themes!
                themes_info.themes = IThemesInfo(data_type).themes


#
# Data types vocabularies
#

@vocabulary_config(name=ALL_DATA_TYPES_VOCABULARY)
class AllTypedSharedToolDataTypesVocabulary(SimpleVocabulary):
    """Vocabulary consolidating data types of all shared tools"""

    def __init__(self, context):
        terms = []
        request = check_request()
        for name, tool in get_utilities_for(ISharedTool):
            if not name:
                continue
            manager = ITypedDataManager(tool, None)
            if manager is None:
                continue
            terms.extend([
                SimpleTerm(datatype.__name__,
                           title=II18n(datatype).query_attribute('label', request=request))
                for datatype in manager.values()
            ])
        terms.sort(key=lambda x: x.title)
        super().__init__(terms)

    def getTermByToken(self, token):
        try:
            return super().getTermByToken(token)
        except LookupError:
            request = check_request()
            translate = request.localizer.translate
            return SimpleTerm(token,
                              title=translate(_("-- missing value ({}) --")).format(token))


def get_all_data_types(request, context=None, fieldname=None):
    """Get list of all registered data types as JSON object"""
    results = []
    translate = request.localizer.translate
    if context is not None:
        values = getattr(context, fieldname, ()) or ()
    else:
        values = ()
    for name, tool in sorted(get_utilities_for(ISharedTool),
                             key=lambda x: II18n(x[1]).query_attribute('title', request=request)):
        if not name:
            continue
        manager = ITypedDataManager(tool, None)
        if manager is None:
            continue
        terms = [
            {
                'id': datatype.__name__,
                'text': II18n(datatype).query_attribute('label', request=request),
                'selected': datatype.__name__ in values
            }
            for datatype in manager.values()
        ]
        content_factory = tool.shared_content_factory.factory
        results.append({
            'text': translate(content_factory.content_name),
            'disabled': True,
            'children': terms
        })
    return results


@vocabulary_config(name=DATA_TYPES_VOCABULARY)
class TypedSharedToolDataTypesVocabulary(SimpleVocabulary):
    """Typed shared tool data types vocabulary"""

    def __init__(self, context):
        terms = []
        parent = ITypedSharedTool(context, None)
        if parent is not None:
            request = check_request()
            manager = ITypedDataManager(parent)
            terms = [
                SimpleTerm(datatype.__name__,
                           title=II18n(datatype).query_attribute('label', request=request))
                for datatype in manager.values()
            ]
        super().__init__(terms)


@vocabulary_config(name=VISIBLE_DATA_TYPES_VOCABULARY)
class TypedSharedToolVisibleDataTypesVocabulary(SimpleVocabulary):
    """Typed shared tool visible data types vocabulary"""

    def __init__(self, context):
        terms = []
        parent = ITypedSharedTool(context, None)
        if parent is not None:
            request = check_request()
            manager = ITypedDataManager(parent)
            terms = [
                SimpleTerm(datatype.__name__,
                           title=II18n(datatype).query_attribute('label', request=request))
                for datatype in manager.get_visible_items()
            ]
        super().__init__(terms)


@vocabulary_config(name=DATA_TYPE_FIELDS_VOCABULARY)
class TypedSharedToolDataTypesFieldsVocabulary(SimpleVocabulary):
    """Typed shared tool data types fields vocabulary"""

    def __init__(self, context):
        terms = []
        parent = ITypedSharedTool(context, None)
        if (parent is not None) and parent.shared_content_info_factory:
            request = check_request()
            translate = request.localizer.translate
            terms = [
                SimpleTerm(name, title=translate(field.title))
                for name, field in getFieldsInOrder(parent.shared_content_info_factory)
            ]
        super().__init__(terms)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface
from zope.schema import Choice

from pyams_content.shared.common.interfaces import ISharedContent, ISharedToolPortalContext, \
    IWfSharedContentPortalContext
from pyams_content.shared.common.interfaces.types import VISIBLE_DATA_TYPES_VOCABULARY
from pyams_sequence.interfaces import IInternalReferencesList

__docformat__ = 'restructuredtext'

from pyams_content import _


TOPIC_CONTENT_TYPE = 'topic'
TOPIC_CONTENT_NAME = _("Topic")


class IWfTopic(IWfSharedContentPortalContext, IInternalReferencesList):
    """Topic interface"""

    data_type = Choice(title=_("Data type"),
                       description=_("Type of content data"),
                       required=False,
                       vocabulary=VISIBLE_DATA_TYPES_VOCABULARY)


class ITopic(ISharedContent):
    """Workflow managed topic interface"""


class ITopicManager(ISharedToolPortalContext):
    """Topic manager interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.portlet.skin.interfaces module

This module provides illustration portlet renderers settings interfaces.
"""

from zope.interface import Interface
from zope.schema import Bool

from pyams_skin.schema import BootstrapThumbnailsSelectionField


__docformat__ = 'restructuredtext'

from pyams_content import _


class IIllustrationPortletBaseRendererSettings(Interface):
    """Illustration portlet base renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Image selection"),
        description=_("Illustration can use responsive selections, but you "
                      "can also force selection of another specific format"),
        default_width=12,
        change_width=False,
        required=False)


class IIllustrationPortletDefaultRendererSettings(IIllustrationPortletBaseRendererSettings):
    """Illustration portlet default renderer settings interface"""

    display_title = Bool(title=_("Display image title"),
                         description=_("Select 'no' to hide illustration title"),
                         required=True,
                         default=True)

    display_author = Bool(title=_("Display author"),
                          description=_("Select 'no' to hide illustration author name"),
                          required=True,
                          default=True)

    display_description = Bool(title=_("Display description"),
                               description=_("Select 'no' to hide image description"),
                               required=True,
                               default=True)


class IIllustrationPortletSideRendererSettings(IIllustrationPortletBaseRendererSettings):
    """Illustration portlet side renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Image selection"),
        description=_("Illustration can use responsive selections, but you "
                      "can also force selection of another specific format"),
        default_width=5,
        required=False)

    zoom_on_click = Bool(title=_("Zoom on click"),
                         description=_("If 'yes', a click on illustration thumbnail will open a "
                                       "modal window to display image"),
                         required=True,
                         default=True)

    def get_css_cols(self, side):
        """Get columns width matching current configuration"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation module

This module defines base navigation menus components.
"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.interface import alsoProvides
from zope.lifecycleevent.interfaces import IObjectAddedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.association import IAssociationContainer
from pyams_content.component.association.container import AssociationContainer
from pyams_content.component.association.interfaces import ASSOCIATION_CONTAINER_KEY
from pyams_content.component.illustration import ILinkIllustrationTarget
from pyams_content.component.links import IInternalLink
from pyams_content.feature.navigation.interfaces import IDynamicMenu, IMenu, IMenuLinksContainer, \
    IMenuLinksContainerTarget, IMenusContainer, IMenusContainerTarget, MENUS_CONTAINER_KEY, \
    MENU_ICON_CLASS, MENU_ICON_HINT
from pyams_content.reference.pictogram import IPictogramTable
from pyams_content.shared.site.interfaces import ISiteContainer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_sequence.reference import InternalReferenceMixin
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import query_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.url import canonical_url, relative_url
from pyams_utils.zodb import volatile_property
from pyams_workflow.interfaces import IWorkflowPublicationInfo
from pyams_zmi.interfaces import IAdminLayer


@factory_config(IMenuLinksContainer)
class MenuLinksContainer(AssociationContainer):
    """Menu links container"""


@adapter_config(required=IMenuLinksContainerTarget,
                provides=IMenuLinksContainer)
def menu_links_container_factory(context):
    """Menu links container factory"""
    return get_annotation_adapter(context, ASSOCIATION_CONTAINER_KEY, IMenuLinksContainer,
                                  name='++ass++')


@subscriber(IObjectAddedEvent, parent_selector=IMenuLinksContainer)
def handle_added_navigation_link(event):
    alsoProvides(event.object, ILinkIllustrationTarget)


@factory_config(IMenu)
class Menu(InternalReferenceMixin, Persistent, Contained):
    """Navigation menu"""

    icon_class = MENU_ICON_CLASS
    icon_hint = MENU_ICON_HINT

    visible = FieldProperty(IMenu['visible'])
    title = FieldProperty(IMenu['title'])
    reference = FieldProperty(IMenu['reference'])
    dynamic_menu = FieldProperty(IMenu['dynamic_menu'])
    force_canonical_url = FieldProperty(IMenu['force_canonical_url'])
    _pictogram_name = FieldProperty(IMenu['pictogram_name'])

    @property
    def links(self):
        """Menu links getter"""
        return IMenuLinksContainer(self)

    @property
    def pictogram_name(self):
        """Pictogram name getter"""
        return self._pictogram_name

    @pictogram_name.setter
    def pictogram_name(self, value):
        """Pictogram name setter"""
        if value != self._pictogram_name:
            self._pictogram_name = value
            del self.pictogram

    @volatile_property
    def pictogram(self):
        """Pictogram getter"""
        table = query_utility(IPictogramTable)
        if table is not None:
            return table.get(self._pictogram_name)
        return None

    def is_visible(self, request=None):
        """Menu visibility checker"""
        if not self.reference:
            return True
        target = self.get_target()
        if target is not None:
            publication_info = IWorkflowPublicationInfo(target, None)
            if publication_info is not None:
                return publication_info.is_visible(request)
        return False

    def get_visible_items(self, request=None):
        """Visible items iterator getter"""
        if self.dynamic_menu and ISiteContainer.providedBy(self.target):
            for item in filter(None,
                               map(lambda x: IDynamicMenu(x, None),
                                   self.target.get_visible_items(request))):
                if IInternalLink.providedBy(item):
                    item.force_canonical_url = self.force_canonical_url
                yield item
        yield from IMenuLinksContainer(self).get_visible_items(request)

    def get_url(self, request=None, view_name=None):
        """Target URL getter"""
        target = self.get_target()
        if target is not None:
            if request is None:
                request = check_request()
            if self.force_canonical_url:
                return canonical_url(target, request, view_name=view_name)
            return relative_url(target, request, view_name=view_name)
        return None


@adapter_config(required=IMenu,
                provides=IViewContextPermissionChecker)
class MenuPermissionChecker(ContextAdapter):
    """Menu permission checker"""

    @property
    def edit_permission(self):
        """Edit permission getter"""
        parent = get_parent(self.context, IMenusContainer)
        return IViewContextPermissionChecker(parent).edit_permission


@factory_config(IMenusContainer)
class MenusContainer(AssociationContainer):
    """Associations menus container"""

    def get_visible_items(self, request=None):
        """Visible items iterator"""
        for menu in filter(lambda x: IMenu(x).visible, self.values()):
            if IAdminLayer.providedBy(request) or menu.is_visible(request):
                yield menu


@adapter_config(required=IMenusContainer,
                provides=IViewContextPermissionChecker)
class MenusContainerPermissionChecker(ContextAdapter):
    """Menus container permission checker"""

    @property
    def edit_permission(self):
        """Edit permission getter"""
        parent = get_parent(self.context, IMenusContainerTarget)
        return IViewContextPermissionChecker(parent).edit_permission


@adapter_config(required=IMenusContainerTarget,
                provides=IMenusContainer)
@adapter_config(name='menus',
                required=IMenusContainerTarget,
                provides=IAssociationContainer)
def menus_container_factory(target):
    """Navigation menus container factory"""
    return get_annotation_adapter(target, MENUS_CONTAINER_KEY, IMenusContainer,
                                  name='++ass++menus')



#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS content.include module

This module is used for Pyramid integration
"""

import re

from zope.interface import classImplements

from pyams_content.component.extfile.interfaces import IExtFileManagerTarget
from pyams_content.component.thesaurus import ICollectionsManagerTarget, ITagsManagerTarget, \
    IThemesManagerTarget
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.interfaces import COMMENT_CONTENT_PERMISSION, CONTRIBUTOR_ROLE, \
    CREATE_CONTENT_PERMISSION, CREATE_VERSION_PERMISSION, GUEST_ROLE, MANAGER_ROLE, \
    MANAGE_CONTENT_PERMISSION, MANAGE_REFERENCE_TABLE_PERMISSION, MANAGE_SITE_PERMISSION, MANAGE_SITE_ROOT_PERMISSION, \
    MANAGE_SITE_TREE_PERMISSION, MANAGE_TOOL_PERMISSION, OID_ACCESS_PATH, OID_ACCESS_ROUTE, OPERATOR_ROLE, OWNER_ROLE, \
    PILOT_ROLE, PUBLISH_CONTENT_PERMISSION, READER_ROLE, REFERENCE_MANAGER_ROLE, WEBMASTER_ROLE
from pyams_layer.interfaces import MANAGE_SKIN_PERMISSION
from pyams_security.interfaces.base import MANAGE_PERMISSION, MANAGE_ROLES_PERMISSION, \
    PUBLIC_PERMISSION, ROLE_ID, VIEW_PERMISSION, VIEW_SYSTEM_PERMISSION
from pyams_security.interfaces.names import ADMIN_USER_ID, SYSTEM_ADMIN_ROLE
from pyams_site.site import BaseSiteRoot
from pyams_thesaurus.interfaces import ADMIN_THESAURUS_PERMISSION, CREATE_THESAURUS_PERMISSION, \
    MANAGE_THESAURUS_CONTENT_PERMISSION, MANAGE_THESAURUS_EXTRACT_PERMISSION

__docformat__ = 'restructuredtext'

from pyams_content import _


def include_package(config):
    """Pyramid package include"""

    # add translations
    config.add_translation_dirs('pyams_content:locales')

    # register permissions
    config.register_permission({
        'id': MANAGE_SITE_ROOT_PERMISSION,
        'title': _("Manage main site root properties")
    })
    config.register_permission({
        'id': MANAGE_REFERENCE_TABLE_PERMISSION,
        'title': _("Manage references table")
    })
    config.register_permission({
        'id': MANAGE_SITE_TREE_PERMISSION,
        'title': _("Manage first level site tree elements")
    })
    config.register_permission({
        'id': MANAGE_SITE_PERMISSION,
        'title': _("Manage site, blog or hub properties")
    })
    config.register_permission({
        'id': MANAGE_TOOL_PERMISSION,
        'title': _("Manage shared tool properties")
    })
    config.register_permission({
        'id': CREATE_CONTENT_PERMISSION,
        'title': _("Create new content")
    })
    config.register_permission({
        'id': CREATE_VERSION_PERMISSION,
        'title': _("Create new version of existing content")
    })
    config.register_permission({
        'id': MANAGE_CONTENT_PERMISSION,
        'title': _("Manage content properties")
    })
    config.register_permission({
        'id': COMMENT_CONTENT_PERMISSION,
        'title': _("Comment existing content")
    })
    config.register_permission({
        'id': PUBLISH_CONTENT_PERMISSION,
        'title': _("Publish or retire existing content")
    })

    # upgrade system manager roles
    config.upgrade_role(SYSTEM_ADMIN_ROLE,
                        permissions={
                            MANAGE_SITE_ROOT_PERMISSION, MANAGE_REFERENCE_TABLE_PERMISSION,
                            MANAGE_SITE_TREE_PERMISSION, MANAGE_SITE_PERMISSION,
                            MANAGE_TOOL_PERMISSION, CREATE_CONTENT_PERMISSION,
                            CREATE_VERSION_PERMISSION, MANAGE_CONTENT_PERMISSION,
                            COMMENT_CONTENT_PERMISSION, PUBLISH_CONTENT_PERMISSION
                        })

    # register new roles
    config.register_role({
        'id': WEBMASTER_ROLE,
        'title': _("Webmaster (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION, MANAGE_PERMISSION,
            VIEW_SYSTEM_PERMISSION, MANAGE_ROLES_PERMISSION,
            CREATE_THESAURUS_PERMISSION, ADMIN_THESAURUS_PERMISSION,
            MANAGE_THESAURUS_CONTENT_PERMISSION, MANAGE_THESAURUS_EXTRACT_PERMISSION,
            MANAGE_SITE_ROOT_PERMISSION, MANAGE_REFERENCE_TABLE_PERMISSION,
            MANAGE_SITE_TREE_PERMISSION, MANAGE_SITE_PERMISSION,
            MANAGE_TOOL_PERMISSION, CREATE_CONTENT_PERMISSION,
            CREATE_VERSION_PERMISSION, MANAGE_CONTENT_PERMISSION,
            COMMENT_CONTENT_PERMISSION, PUBLISH_CONTENT_PERMISSION,
            MANAGE_SKIN_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE)
        }
    })
    config.register_role({
        'id': REFERENCE_MANAGER_ROLE,
        'title': _("References manager (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION, VIEW_SYSTEM_PERMISSION,
            MANAGE_REFERENCE_TABLE_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(WEBMASTER_ROLE)
        }
    })
    config.register_role({
        'id': PILOT_ROLE,
        'title': _("Pilot (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION, MANAGE_PERMISSION,
            VIEW_SYSTEM_PERMISSION, MANAGE_ROLES_PERMISSION,
            MANAGE_SITE_PERMISSION, MANAGE_TOOL_PERMISSION,
            MANAGE_CONTENT_PERMISSION, COMMENT_CONTENT_PERMISSION,
            PUBLISH_CONTENT_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(WEBMASTER_ROLE)
        }
    })
    config.register_role({
        'id': MANAGER_ROLE,
        'title': _("Manager (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION, MANAGE_PERMISSION,
            VIEW_SYSTEM_PERMISSION, MANAGE_CONTENT_PERMISSION,
            CREATE_VERSION_PERMISSION, COMMENT_CONTENT_PERMISSION,
            PUBLISH_CONTENT_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(WEBMASTER_ROLE),
            ROLE_ID.format(PILOT_ROLE)
        }
    })
    config.register_role({
        'id': OWNER_ROLE,
        'title': _("Owner (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION, MANAGE_PERMISSION,
            VIEW_SYSTEM_PERMISSION, MANAGE_ROLES_PERMISSION,
            MANAGE_CONTENT_PERMISSION, CREATE_VERSION_PERMISSION,
            COMMENT_CONTENT_PERMISSION
        }
    })
    config.register_role({
        'id': CONTRIBUTOR_ROLE,
        'title': _("Contributor (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION, MANAGE_PERMISSION,
            VIEW_SYSTEM_PERMISSION, CREATE_CONTENT_PERMISSION,
            MANAGE_CONTENT_PERMISSION, CREATE_VERSION_PERMISSION,
            COMMENT_CONTENT_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(WEBMASTER_ROLE),
            ROLE_ID.format(PILOT_ROLE),
            ROLE_ID.format(OWNER_ROLE)
        }
    })
    config.register_role({
        'id': READER_ROLE,
        'title': _("Reader (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION, MANAGE_PERMISSION,
            VIEW_SYSTEM_PERMISSION, COMMENT_CONTENT_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(WEBMASTER_ROLE),
            ROLE_ID.format(PILOT_ROLE),
            ROLE_ID.format(MANAGER_ROLE),
            ROLE_ID.format(OWNER_ROLE),
            ROLE_ID.format(CONTRIBUTOR_ROLE)
        }
    })
    config.register_role({
        'id': OPERATOR_ROLE,
        'title': _("Operator (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION, VIEW_SYSTEM_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE)
        }
    })
    config.register_role({
        'id': GUEST_ROLE,
        'title': _("Guest user (role)"),
        'permissions': {
            PUBLIC_PERMISSION, VIEW_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(WEBMASTER_ROLE),
            ROLE_ID.format(PILOT_ROLE),
            ROLE_ID.format(MANAGER_ROLE),
            ROLE_ID.format(OWNER_ROLE),
            ROLE_ID.format(CONTRIBUTOR_ROLE)
        }
    })

    # site root extensions
    classImplements(BaseSiteRoot, IExtFileManagerTarget)
    classImplements(BaseSiteRoot, ITagsManagerTarget)
    classImplements(BaseSiteRoot, IThemesManagerTarget)
    classImplements(BaseSiteRoot, ICollectionsManagerTarget)
    classImplements(BaseSiteRoot, IPreviewTarget)

    # custom routes
    config.add_route(OID_ACCESS_ROUTE,
                     config.registry.settings.get(f'{OID_ACCESS_ROUTE}_route.path',
                                                  OID_ACCESS_PATH))

    try:
        import pyams_zmi  # pylint: disable=import-outside-toplevel,unused-import
    except ImportError:
        config.scan(ignore=[re.compile(r'pyams_content\..*\.zmi\.?.*').search])
    else:
        config.scan()



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.paragraph module

"""

from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration import BasicIllustration
from pyams_content.component.illustration.interfaces import IIllustrationParagraph, \
    ILLUSTRATION_PARAGRAPH_ICON_CLASS, ILLUSTRATION_PARAGRAPH_NAME, \
    ILLUSTRATION_PARAGRAPH_RENDERERS, ILLUSTRATION_PARAGRAPH_TYPE
from pyams_content.component.paragraph import BaseParagraph
from pyams_content.component.paragraph.interfaces import IBaseParagraph
from pyams_content.feature.renderer import RenderersVocabulary
from pyams_utils.factory import factory_config
from pyams_utils.vocabulary import vocabulary_config


__docformat__ = 'restructuredtext'


@factory_config(IIllustrationParagraph)
@factory_config(IBaseParagraph, name=ILLUSTRATION_PARAGRAPH_TYPE)
class IllustrationParagraph(BasicIllustration, BaseParagraph):
    """Illustration paragraph persistent class"""

    factory_name = ILLUSTRATION_PARAGRAPH_TYPE
    factory_label = ILLUSTRATION_PARAGRAPH_NAME
    factory_intf = IIllustrationParagraph

    icon_class = ILLUSTRATION_PARAGRAPH_ICON_CLASS

    description = FieldProperty(IIllustrationParagraph['description'])
    renderer = FieldProperty(IIllustrationParagraph['renderer'])


@vocabulary_config(name=ILLUSTRATION_PARAGRAPH_RENDERERS)
class IllustrationParagraphRenderersVocabulary(RenderersVocabulary):
    """Illustration paragraph renderers vocabulary"""

    content_interface = IIllustrationParagraph



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.paragraph module

"""

from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.gallery import BaseGallery
from pyams_content.component.gallery.interfaces import GALLERY_PARAGRAPH_ICON_CLASS, \
    GALLERY_PARAGRAPH_NAME, GALLERY_PARAGRAPH_RENDERERS, GALLERY_PARAGRAPH_TYPE, IGalleryParagraph
from pyams_content.component.paragraph import BaseParagraph, IBaseParagraph
from pyams_content.feature.renderer import RenderersVocabulary
from pyams_utils.factory import factory_config
from pyams_utils.vocabulary import vocabulary_config


__docformat__ = 'restructuredtext'


@factory_config(IGalleryParagraph)
@factory_config(IBaseParagraph, name=GALLERY_PARAGRAPH_TYPE)
class GalleryParagraph(BaseGallery, BaseParagraph):
    """Gallery class"""

    factory_name = GALLERY_PARAGRAPH_TYPE
    factory_label = GALLERY_PARAGRAPH_NAME
    factory_intf = IGalleryParagraph

    icon_class = GALLERY_PARAGRAPH_ICON_CLASS

    renderer = FieldProperty(IGalleryParagraph['renderer'])


@vocabulary_config(name=GALLERY_PARAGRAPH_RENDERERS)
class GalleryParagraphRenderersVocabulary(RenderersVocabulary):
    """Gallery paragraph renderers vocabulary"""

    content_interface = IGalleryParagraph



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.portlet.skin module

This module defines gallery portlet renderers.
"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.gallery import IGallery
from pyams_content.component.gallery.portlet import IGalleryPortletSettings
from pyams_content.component.gallery.portlet.skin.interfaces import \
    IGalleryPortletCarouselRendererSettings, IGalleryPortletDefaultRendererSettings
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.list import random_iter

__docformat__ = 'restructuredtext'

from pyams_content import _


class BaseGalleryPortletRenderer(PortletRenderer):
    """Base gallery portlet renderer"""

    def get_visible_medias(self):
        """Visible medias getter"""
        settings = self.settings
        gallery = IGallery(self.context, None)
        if gallery is not None:
            yield from gallery.get_visible_medias()
        yield from settings.get_visible_medias()


#
# Default gallery portlet renderer
#

@factory_config(provided=IGalleryPortletDefaultRendererSettings)
class GalleryPortletDefaultRendererSettings(Persistent, Contained):
    """Gallery portlet default renderer settings"""

    thumb_selection = FieldProperty(IGalleryPortletDefaultRendererSettings['thumb_selection'])

    def get_css_cols(self):
        """Get CSS cols matching current selections"""
        return ' '.join([
            f"col-{'{}-'.format(device) if device != 'xs' else ''}{selection.cols}"
            for device, selection in self.thumb_selection.items()
        ])


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface, IGalleryPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/gallery-default.pt', layer=IPyAMSLayer)
class GalleryPortletDefaultRenderer(BaseGalleryPortletRenderer):
    """Gallery portlet default renderer"""

    label = _("Grid gallery renderer (default)")

    settings_interface = IGalleryPortletDefaultRendererSettings
    weight = 10


#
# Carousel gallery portlet renderer
#

@factory_config(provided=IGalleryPortletCarouselRendererSettings)
class GalleryPortletCarouselRendererSettings(Persistent, Contained):
    """Gallery portlet carousel renderer settings"""

    thumb_selection = FieldProperty(IGalleryPortletCarouselRendererSettings['thumb_selection'])


@adapter_config(name='carousel',
                required=(IPortalContext, IPyAMSLayer, Interface, IGalleryPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/gallery-carousel.pt', layer=IPyAMSLayer)
class GalleryPortletCarouselRenderer(BaseGalleryPortletRenderer):
    """Gallery carousel default renderer"""

    label = _("Carousel gallery renderer")

    settings_interface = IGalleryPortletCarouselRendererSettings
    weight = 20


#
# Random image gallery portlet renderer
#

@factory_config(provided=IGalleryPortletCarouselRendererSettings)
class GalleryPortletRandomImageRendererSettings(Persistent, Contained):
    """Gallery portlet random image renderer settings"""

    thumb_selection = FieldProperty(IGalleryPortletCarouselRendererSettings['thumb_selection'])


@adapter_config(name='random',
                required=(IPortalContext, IPyAMSLayer, Interface, IGalleryPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/gallery-random.pt', layer=IPyAMSLayer)
class GalleryPortletRandomImageRenderer(BaseGalleryPortletRenderer):
    """Gallery carousel random image renderer"""

    label = _("Random image renderer")

    settings_interface = IGalleryPortletCarouselRendererSettings
    weight = 30

    def get_media(self):
        """Visible media getter"""
        return next(random_iter(self.settings.get_visible_medias()))



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.dashboard module

This module provides dashboard management components which are common to all
shared contents.
"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Any, Eq, Or
from pyramid.decorator import reify
from pyramid.events import subscriber
from pyramid.interfaces import IView
from zope.annotation import IAttributeAnnotatable
from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import Interface, implementer
from zope.intid import IIntIds
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_catalog.query import CatalogResultSet
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION, PUBLISH_CONTENT_PERMISSION
from pyams_content.shared.common.interfaces import IBaseSharedTool, IManagerRestrictions, ISharedContent, \
    IWfSharedContentRoles, SHARED_CONTENT_TYPES_VOCABULARY
from pyams_content.shared.common.zmi.content import shared_content_version_getter
from pyams_content.zmi.interfaces import IAllDashboardMenu, IDashboardColumn, \
    IDashboardContentModifier, IDashboardContentNumber, IDashboardContentOwner, \
    IDashboardContentStatus, IDashboardContentStatusDatetime, IDashboardContentTimestamp, \
    IDashboardContentVersion, IDashboardTable, IDashboardView, IMyDashboardMenu
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.utility import get_principal
from pyams_sequence.interfaces import ISequentialIdInfo, ISequentialIdTarget, ISequentialIntIds
from pyams_skin.viewlet.help import AlertMessage
from pyams_table.interfaces import ITableRowUpdatedEvent, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.date import SH_DATETIME_FORMAT, format_datetime
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.list import boolean_iter, unique_iter
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import ViewContentProvider, viewlet_config
from pyams_workflow.interfaces import IWorkflow, IWorkflowPublicationInfo, \
    IWorkflowPublicationSupport, IWorkflowState, IWorkflowVersions
from pyams_workflow.versions import get_last_version_in_state
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import IInnerTable, ITableWithActions
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IMenuHeader
from pyams_zmi.table import InnerTableAdminView, MultipleTablesAdminView, Table, TableAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@subscriber(ITableRowUpdatedEvent,
            context_selector=IDashboardTable,
            row_context_selector=ISharedContent)
def handle_shared_content_table_row_update(event):
    """Shared content table row update event handler"""
    item_key = ICacheKeyValue(event.item)
    versions = IWorkflowVersions(event.context, None)
    if versions is not None:
        event.object.rows_state[item_key] = versions.get_last_versions()[-1]


@adapter_config(required=(ISequentialIdTarget, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentNumber)
def sequence_target_number(context, request, column):
    """Sequence target number getter"""
    sequence_info = ISequentialIdInfo(context, None)
    if sequence_info is not None:
        sequence = get_utility(ISequentialIntIds, name=context.sequence_name)
        return sequence.get_base_oid(sequence_info.oid, context.sequence_prefix)
    return None


@adapter_config(required=(IWorkflowPublicationSupport, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentStatus)
def content_workflow_status(context, request, column):
    """Content workflow status getter"""
    state = IWorkflowState(context, None)
    if state is not None:
        workflow = IWorkflow(context)
        result = request.localizer.translate(workflow.get_state_label(state.state))
        if state.state in workflow.waiting_states:
            if state.state_urgency:
                result += ' <i class="fas fa-fw fa-exclamation-triangle text-danger"></i>'
        elif state.state in workflow.published_states:
            pub_info = IWorkflowPublicationInfo(context, None)
            if (pub_info is not None) and not pub_info.is_published():
                translate = request.localizer.translate
                result += ' <i class="fas fa-fw fa-hourglass-half opacity-75 hint align-base" ' \
                          '    data-offset="5" title="{}"></i>'.format(
                    translate(_("Content publication start date is not passed yet")))
        return result
    return None


@adapter_config(required=(ISharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentStatus)
def shared_content_workflow_status(context, request, view):
    """Shared content workflow status getter"""
    return shared_content_version_getter(context, request, view, IDashboardContentStatus)


@adapter_config(required=(IWorkflowPublicationSupport, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentStatusDatetime)
def content_workflow_status_datetime(context, request, column):
    """Content workflow status datetime getter"""
    state = IWorkflowState(context, None)
    if state is not None:
        return format_datetime(state.state_date, SH_DATETIME_FORMAT,
                               request=request)
    return None


@adapter_config(required=(ISharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentStatusDatetime)
def shared_content_workflow_status_datetime(context, request, view):
    """Shared content workflow status datetime getter"""
    return shared_content_version_getter(context, request, view, IDashboardContentStatusDatetime)


@adapter_config(required=(IWorkflowPublicationSupport, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentVersion)
def content_workflow_version(context, request, column):
    """Content workflow version getter"""
    state = IWorkflowState(context, None)
    if state is not None:
        return str(state.version_id)
    return None


@adapter_config(required=(ISharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentVersion)
def shared_content_workflow_version(context, request, view):
    """Shared content workflow version getter"""
    return shared_content_version_getter(context, request, view, IDashboardContentVersion)


@adapter_config(required=(IWorkflowPublicationSupport, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentModifier)
def content_workflow_status_principal(context, request, column):
    """Content workflow status principal getter"""
    state = IWorkflowState(context, None)
    if state is not None:
        manager = get_utility(ISecurityManager)
        return manager.get_principal(state.state_principal).title
    return None


@adapter_config(required=(ISharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentModifier)
def shared_content_workflow_status_principal(context, request, view):
    """Shared content workflow status principal getter"""
    return shared_content_version_getter(context, request, view, IDashboardContentModifier)


@adapter_config(required=(IWorkflowPublicationSupport, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentOwner)
def content_workflow_owner(context, request, column):
    """Content workflow owner getter"""
    try:
        owner = IWfSharedContentRoles(context).owner
    except (TypeError, AttributeError):
        return None
    if owner:
        return get_principal(request, next(iter(owner))).title
    return None


@adapter_config(required=(ISharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentOwner)
def shared_content_workflow_owner(context, request, view):
    """Shared content workflow owner getter"""
    return shared_content_version_getter(context, request, view, IDashboardContentOwner)


@adapter_config(required=(IAttributeAnnotatable, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentTimestamp)
def content_timestamp(context, request, column):
    """Content timestamp getter"""
    dc = IZopeDublinCore(context, None)
    if dc is not None:
        return format_datetime(tztime(dc.modified), SH_DATETIME_FORMAT,
                               request=request)
    return None


@adapter_config(required=(ISharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentTimestamp)
def shared_content_timestamp(context, request, view):
    """Shared content timestamp getter"""
    return shared_content_version_getter(context, request, view, IDashboardContentTimestamp)


#
# Base shared tool dashboard components
#

class BaseDashboardTable(Table):
    """Base dashboard table"""

    @property
    def sort_index(self):
        return len(self.columns) - 1

    sort_order = 'desc'

    @reify
    def values(self):
        return list(super().values)

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        attributes.setdefault('table', {}).update({
            'data-ams-order': f'{self.sort_index},{self.sort_order}'
        })
        return attributes


@implementer(IDashboardTable)
class DashboardTable(BaseDashboardTable):
    """Dashboard table"""


@implementer(ITableWithActions)
class DashboardTableWithActions(DashboardTable):
    """Dashboard table with actions"""


@implementer(IDashboardView)
class SharedToolDashboardViewMixin:
    """Shared tool dashboard mixin view"""

    empty_label = None
    single_label = None
    plural_label = None

    @property
    def table_label(self):
        translate = self.request.localizer.translate
        length = len(self.table.values)
        if length == 0:
            return translate(self.empty_label)
        if length == 1:
            return translate(self.single_label)
        return translate(self.plural_label).format(length)


class BaseSharedToolDashboardView(SharedToolDashboardViewMixin, InnerTableAdminView,
                                  ViewContentProvider):
    """Base shared tool dashboard view with multiple tables"""

    hide_section = True

    def render(self):
        if not self.table.values:
            return ''
        return super().render()


class BaseSharedToolDashboardSingleView(SharedToolDashboardViewMixin, TableAdminView):
    """Base shared tool dashboard view with single table"""


#
# Main shared tool dashboard
#

@adapter_config(required=(IBaseSharedTool, IAdminLayer, Interface, IContentManagementMenu),
                provides=IMenuHeader)
def shared_tool_menu_header(context, request, view, menu):
    """Shared tool menu header"""
    return request.localizer.translate(_("Users activity"))


@viewlet_config(name='dashboard.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IContentManagementMenu, weight=5,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolDashboardMenu(NavigationMenuItem):
    """Shared tool dashboard menu"""

    label = _("Dashboard")
    icon_class = 'fas fa-chart-line'
    href = '#dashboard.html'


@pagelet_config(name='dashboard.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolDashboardView(MultipleTablesAdminView):
    """Shared tool dashboard view"""

    header_label = _("My dashboard")
    table_label = _("My dashboard")


@adapter_config(name='no-content-warning',
                required=(IBaseSharedTool, IAdminLayer, SharedToolDashboardView),
                provides=IInnerTable, force_implements=False)
class SharedToolMissingContentWarning(AlertMessage):
    """Shared tool missing content warning"""

    def __init__(self, context, request, view):
        super().__init__(context, request, view, None)

    _message = _("You are not actually concerned by any content.")

    def render(self):
        for view in self.view.tables:
            if not IInnerTable.providedBy(view):
                continue
            has_values, _values = boolean_iter(view.table.values)
            if has_values:
                return ''
        return super().render()

    weight = 999


#
# Manager waiting contents
#

class SharedToolDashboardManagerWaitingTable(DashboardTable):
    """Shared tool dashboard manager waiting table"""

    @reify
    def id(self):
        return f'{super().id}_waiting'


@adapter_config(name='manager-waiting',
                required=(IBaseSharedTool, IAdminLayer, SharedToolDashboardView),
                provides=IInnerTable)
class SharedToolDashboardManagerWaitingView(BaseSharedToolDashboardView):
    """Shared tool dashboard manager waiting view"""

    table_class = SharedToolDashboardManagerWaitingTable

    empty_label = _("MANAGER - No content waiting for your action")
    single_label = _("MANAGER - 1 content waiting for your action")
    plural_label = _("MANAGER - {} contents waiting for your action")

    weight = 20


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolDashboardManagerWaitingTable),
                provides=IValues)
class SharedToolDashboardManagerWaitingValues(ContextRequestViewAdapter):
    """Shared tool dashboard waiting values getter"""

    @property
    def values(self):
        """Table values getter"""
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        workflow = IWorkflow(self.context)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(self.context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()),
                     Any(catalog['workflow_state'], workflow.waiting_states))
        yield from filter(
            self.check_access,
            unique_iter(
                map(get_last_version_in_state,
                    CatalogResultSet(CatalogQuery(catalog).query(params,
                                                                 sort_index='modified_date')))))

    def check_access(self, content):
        """Content access checker"""
        if self.request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context=content):
            return True
        roles = IWfSharedContentRoles(content)
        if self.request.principal.id in roles.managers:
            return True
        restrictions = IManagerRestrictions(content)
        if restrictions is not None:
            return restrictions.can_access(content, PUBLISH_CONTENT_PERMISSION, self.request)
        return False


#
# Last owned contents waiting for action
#

class SharedToolDashboardOwnerWaitingTable(DashboardTable):
    """Table of owned contents waiting for action"""

    @reify
    def id(self):
        return f'{super().id}_owner_waiting'


@adapter_config(name='owner-waiting',
                required=(IBaseSharedTool, IAdminLayer, SharedToolDashboardView),
                provides=IInnerTable)
class SharedToolDashboardOwnerWaitingView(BaseSharedToolDashboardView):
    """View of owned contents waiting for action"""

    table_class = SharedToolDashboardOwnerWaitingTable

    empty_label = _("CONTRIBUTOR - 0 content waiting for action")
    single_label = _("CONTRIBUTOR - 1 content waiting for action")
    plural_label = _("CONTRIBUTOR - {0} contents waiting for action")

    weight = 30


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolDashboardOwnerWaitingTable),
                provides=IValues)
class SharedToolDashboardOwnerWaitingValues(ContextRequestViewAdapter):
    """Shared tool dashboard waiting owned contents values adapter"""

    @property
    def values(self):
        """Table values getter"""
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        workflow = IWorkflow(self.context)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(self.context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()),
                     Any(catalog['workflow_state'], workflow.waiting_states),
                     Eq(catalog['workflow_principal'], self.request.principal.id))
        yield from unique_iter(
            map(get_last_version_in_state,
                CatalogResultSet(CatalogQuery(catalog).query(params,
                                                             sort_index='modified_date'))))


#
# Last owner modified contents
#

class SharedToolDashboardOwnerModifiedTable(DashboardTable):
    """Shared tool dashboard owner modified table"""

    @reify
    def id(self):
        return f'{super().id}_modified'


@adapter_config(name='owner-modified',
                required=(IBaseSharedTool, IAdminLayer, SharedToolDashboardView),
                provides=IInnerTable)
class SharedToolDashboardOwnerModifiedView(BaseSharedToolDashboardView):
    """Shared tool dashboard owner modified view"""

    table_class = SharedToolDashboardOwnerModifiedTable

    empty_label = _("CONTRIBUTOR - 0 modified content")
    single_label = _("CONTRIBUTOR - 1 modified content")

    @property
    def plural_label(self):
        length = len(self.table.values)
        if length > 50:
            return _("CONTRIBUTOR - Last {} modified contents")
        return _("CONTRIBUTOR - {} modified contents")

    weight = 40


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolDashboardOwnerModifiedTable),
                provides=IValues)
class SharedToolDashboardOwnerModifiedValues(ContextRequestViewAdapter):
    """Shared tool dashboard owner modified adapter"""

    @property
    def values(self):
        """Table values getter"""
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(self.context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()),
                     Or(Eq(catalog['role:owner'], principal_id),
                        Eq(catalog['role:contributor'], principal_id)))
        yield from unique_iter(
            map(get_last_version_in_state,
                CatalogResultSet(CatalogQuery(catalog).query(params,
                                                             limit=50,
                                                             sort_index='modified_date',
                                                             reverse=True))))


#
# All my contents menu
#

@viewletmanager_config(name='my-contents.menu',
                       context=IBaseSharedTool, layer=IAdminLayer,
                       manager=IContentManagementMenu, weight=10,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IMyDashboardMenu)
class SharedToolMyDashboardMenu(NavigationMenuItem):
    """Shared tool 'my contents' dashboard menu"""

    label = _("My contents")
    icon_class = 'fas fa-user'
    href = '#'


#
# My preparations
# Dashboard of owned and modified contents which can be updated
#

@viewlet_config(name='my-preparations.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=5,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolPreparationsMenu(NavigationMenuItem):
    """Shared tool preparations dashboard menu"""

    label = _("My drafts")
    href = '#my-preparations.html'


@implementer(IView)
class SharedToolPreparationsTable(DashboardTable):
    """Shared tool preparations table"""


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolPreparationsTable),
                provides=IValues)
class SharedToolPreparationsValues(ContextRequestViewAdapter):
    """Shared tool preparations values adapter"""

    @property
    def values(self):
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        workflow = IWorkflow(self.context)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(self.context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()),
                     Or(Eq(catalog['role:owner'], principal_id),
                        Eq(catalog['role:contributor'], principal_id)),
                     Eq(catalog['workflow_state'], workflow.initial_state))
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-preparations.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolPreparationsView(BaseSharedToolDashboardSingleView):
    """Shared tool preparations view"""

    header_label = _("My drafts")
    table_class = SharedToolPreparationsTable

    empty_label = _("CONTRIBUTOR - 0 content in preparation")
    single_label = _("CONTRIBUTOR - 1 content in preparation")
    plural_label = _("CONTRIBUTOR - {} contents in preparation")


#
# My submissions
# Dashboard of contents waiting for manager action
#

@viewlet_config(name='my-submissions.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=10,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolSubmissionsMenu(NavigationMenuItem):
    """Shared tool submissions dashboard menu"""

    label = _("My submissions")
    href = '#my-submissions.html'


@implementer(IView)
class SharedToolSubmissionsTable(DashboardTable):
    """Shared tool submissions table"""


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolSubmissionsTable),
                provides=IValues)
class SharedToolSubmissionsValues(ContextRequestViewAdapter):
    """Shared tool submissions values adapter"""

    @property
    def values(self):
        context = self.context
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        workflow = IWorkflow(context)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()),
                     Or(Eq(catalog['role:owner'], principal_id),
                        Eq(catalog['role:contributor'], principal_id)),
                     Any(catalog['workflow_state'], workflow.waiting_states))
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-submissions.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolSubmissionsView(BaseSharedToolDashboardSingleView):
    """Shared tool submissions view"""

    header_label = _("My submissions")
    table_class = SharedToolSubmissionsTable

    empty_label = _("CONTRIBUTOR - 0 submitted content")
    single_label = _("CONTRIBUTOR - 1 submitted content")
    plural_label = _("CONTRIBUTOR - {} submitted contents")


#
# My publications
# Dashboard of published contents
#

@viewlet_config(name='my-publications.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=15,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolPublicationsMenu(NavigationMenuItem):
    """Shared tool publications dashboard menu"""

    label = _("My publications")
    href = '#my-publications.html'


@implementer(IView)
class SharedToolPublicationsTable(DashboardTable):
    """Shared tool publications table"""


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolPublicationsTable),
                provides=IValues)
class SharedToolPublicationsValues(ContextRequestViewAdapter):
    """Shared tool publications values adapter"""

    @property
    def values(self):
        context = self.context
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        workflow = get_utility(IWorkflow, name=context.shared_content_workflow)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()) &
                     Or(Eq(catalog['role:owner'], principal_id),
                        Eq(catalog['role:contributor'], principal_id)),
                     Any(catalog['workflow_state'], workflow.published_states))
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-publications.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolPublicationsView(BaseSharedToolDashboardSingleView):
    """Shared tool publications view"""

    header_label = _("My publications")
    table_class = SharedToolPublicationsTable

    empty_label = _("CONTRIBUTOR - 0 published content")
    single_label = _("CONTRIBUTOR - 1 published content")
    plural_label = _("CONTRIBUTOR - {} published contents")


#
# My retired contents
# Dashboard of retired contents
#

@viewlet_config(name='my-retired-contents.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=20,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolRetiredContentsMenu(NavigationMenuItem):
    """Shared tool retired contents dashboard menu"""

    label = _("My retired contents")
    href = '#my-retired-contents.html'


@implementer(IView)
class SharedToolRetiredContentsTable(DashboardTable):
    """Shared tool retired contents table"""


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolRetiredContentsTable),
                provides=IValues)
class SharedToolRetiredContentsValues(ContextRequestViewAdapter):
    """Shared tool retired contents values adapter"""

    @property
    def values(self):
        context = self.context
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        workflow = get_utility(IWorkflow, name=context.shared_content_workflow)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()) &
                     Or(Eq(catalog['role:owner'], principal_id),
                        Eq(catalog['role:contributor'], principal_id)),
                     Any(catalog['workflow_state'], workflow.retired_states))
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-retired-contents.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolRetiredContentsView(BaseSharedToolDashboardSingleView):
    """Shared tool retired contents view"""

    header_label = _("My retired contents")
    table_class = SharedToolRetiredContentsTable

    empty_label = _("CONTRIBUTOR - 0 retired content")
    single_label = _("CONTRIBUTOR - 1 retired content")
    plural_label = _("CONTRIBUTOR - {} retired contents")


#
# My archived contents
# Dashboard of archived contents
#

@viewlet_config(name='my-archived-contents.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=25,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolArchivedContentsMenu(NavigationMenuItem):
    """Shared tool archived contents dashboard menu"""

    label = _("My archived contents")
    href = '#my-archived-contents.html'


@implementer(IView)
class SharedToolArchivedContentsTable(DashboardTable):
    """Shared tool archived contents table"""


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolArchivedContentsTable),
                provides=IValues)
class SharedToolArchivedContentsValues(ContextRequestViewAdapter):
    """Shared tool archived contents values adapter"""

    @property
    def values(self):
        context = self.context
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        workflow = get_utility(IWorkflow, name=context.shared_content_workflow)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()) &
                     Or(Eq(catalog['role:owner'], principal_id),
                        Eq(catalog['role:contributor'], principal_id)),
                     Any(catalog['workflow_state'], workflow.archived_states))
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-archived-contents.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolArchivedContentsView(BaseSharedToolDashboardSingleView):
    """Shared tool archived contents view"""

    header_label = _("My archived contents")
    table_class = SharedToolArchivedContentsTable

    empty_label = _("CONTRIBUTOR - 0 archived content")
    single_label = _("CONTRIBUTOR - 1 archived content")
    plural_label = _("CONTRIBUTOR - {} archived contents")


#
# All interventions menu
#

@viewletmanager_config(name='all-interventions.menu',
                       context=IBaseSharedTool, layer=IAdminLayer,
                       manager=IContentManagementMenu, weight=20,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IAllDashboardMenu)
class SharedToolAllInterventionsMenu(NavigationMenuItem):
    """Shared tool 'all interventions' dashboard menu"""

    label = _("All interventions")
    css_class = 'open'
    icon_class = 'fas fa-pen-square'
    href = '#'


#
# Last published contents
#

@viewlet_config(name='last-published.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IAllDashboardMenu, weight=25,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolLastPublicationsMenu(NavigationMenuItem):
    """Shared tool modified contents dashboard menu"""

    label = _("Last publications")
    href = '#last-published.html'


@implementer(IView)
class SharedToolLastPublicationsTable(DashboardTable):
    """Shared tool dashboard last publications table"""


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolLastPublicationsTable),
                provides=IValues)
class SharedToolLastPublicationsValues(ContextRequestViewAdapter):
    """Shared tool publications values adapter"""

    @property
    def values(self):
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        workflow = get_utility(IWorkflow, name=self.context.shared_content_workflow)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(self.context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()) &
                     Any(catalog['workflow_state'], workflow.published_states))
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         limit=50,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='last-published.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolLastPublicationsView(BaseSharedToolDashboardSingleView):
    """Shared tool last publications view"""

    header_label = _("Last publications")
    table_class = SharedToolLastPublicationsTable

    empty_label = _("CONTRIBUTORS - 0 published content")
    single_label = _("CONTRIBUTORS - 1 published content")

    @property
    def plural_label(self):
        length = len(self.table.values)
        if length == 50:
            return _("CONTRIBUTORS - Last {} published contents")
        return _("CONTRIBUTORS - {} published contents")


#
# Last modified contents
#

@viewlet_config(name='last-modified.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IAllDashboardMenu, weight=30,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolLastModifiedMenu(NavigationMenuItem):
    """Shared tool modified contents dashboard menu"""

    label = _("Last modifications")
    href = '#last-modified.html'


@implementer(IView)
class SharedToolLastModificationsTable(DashboardTable):
    """Shared tool dashboard last modifications table"""


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedToolLastModificationsTable),
                provides=IValues)
class SharedToolLastModificationsValues(ContextRequestViewAdapter):
    """Shared tool modifications values adapter"""

    @property
    def values(self):
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(self.context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()))
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         limit=50,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='last-modified.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolLastModificationsView(BaseSharedToolDashboardSingleView):
    """Shared tool last modifications view"""

    header_label = _("Last modifications")
    table_class = SharedToolLastModificationsTable

    empty_label = _("CONTRIBUTORS - 0 modified content")
    single_label = _("CONTRIBUTORS - 1 modified content")

    @property
    def plural_label(self):
        length = len(self.table.values)
        if length == 50:
            return _("CONTRIBUTORS - Last {} modified contents")
        return _("CONTRIBUTORS - {} modified contents")



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.skin.title module

This module defines components which are used for shared contents title rendering.
"""

from zope.interface import Interface

from pyams_content.interfaces import IBaseContent
from pyams_content.skin.interfaces import IContentTitle
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config

__docformat__ = 'restructuredtext'


@adapter_config(required=(IBaseContent, IPyAMSUserLayer, Interface),
                provides=IContentTitle)
def base_content_title(context, request, view):
    """Base content title adapter"""
    return II18n(context).query_attribute('title', request=request)


@contentprovider_config(name='pyams_content.title',
                        layer=IPyAMSUserLayer, view=Interface)
class SharedContentTitleContentProvider(ViewContentProvider):
    """Shared content title content provider"""

    def render(self, template_name=''):
        return self.request.registry.queryMultiAdapter((self.context, self.request, self.view),
                                                       IContentTitle) or ''



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.summary module

This module provides components which are used to display a summary of main content properties.
"""

from zope.interface import Interface

from pyams_content.shared.common import IWfSharedContent, IWfSharedContentRoles
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces import DISPLAY_MODE, IDataConverter
from pyams_form.interfaces.form import IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.utility import get_principal
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_skin.viewlet.actions import ContextAction
from pyams_utils.adapter import adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.timezone import tztime
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflow, IWorkflowPublicationInfo, IWorkflowState, \
    IWorkflowStateHistoryItem
from pyams_zmi.form import AdminModalDisplayForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='summary.action',
                context=IWfSharedContent, layer=IAdminLayer, view=Interface,
                manager=IToolbarViewletManager, weight=5,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedContentSummaryAction(ContextAction):
    """Shared content summary action"""

    status = 'transparent'
    css_class = 'btn-xs rounded-circle border-primary mx-1'
    icon_class = 'fas fa-info text-primary px-0 py-1'

    hint = _("Content information")

    href = 'summary.html'
    modal_target = True


@ajax_form_config(name='summary.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class SharedContentSummaryView(AdminModalDisplayForm):
    """Shared content summary view"""

    legend = _("Display content summary")
    modal_class = 'modal-xl'


@adapter_config(name='dublincore',
                required=(IWfSharedContent, IAdminLayer, SharedContentSummaryView),
                provides=IGroup)
class SharedContentDublinCoreGroup(Group):
    """Shared content dublin-core group"""

    legend = _("Identity card")
    weight = 10

    fields = Fields(IWfSharedContent).select('title') + \
        Fields(ISequentialIdInfo).select('public_oid')
    mode = DISPLAY_MODE


@adapter_config(name='workflow-waiting',
                required=(IWfSharedContent, IAdminLayer, SharedContentSummaryView),
                provides=IGroup)
class SharedContentWorkflowWaitingState(Group):
    """Shared content workflow"""

    def __new__(cls, context, request, form):
        state  = IWorkflowState(context, None)
        if state is None:
            return None
        workflow = IWorkflow(context)
        if state.state not in workflow.waiting_states:
            return None
        return Group.__new__(cls)

    legend = _("Workflow requested action")
    weight = 20

    fields = Fields(IWorkflowState).select('state', 'state_urgency') + \
        Fields(IWorkflowStateHistoryItem).select('comment')
    mode = DISPLAY_MODE

    ignore_context = True

    def update_widgets(self, prefix=None, use_form_mode=True):
        self.parent_form.widgets.ignore_context = True
        super().update_widgets(prefix, use_form_mode)
        state = IWorkflowState(self.context)
        state_widget = self.widgets.get('state')
        if state_widget is not None:
            translate = self.request.localizer.translate
            workflow = IWorkflow(self.context)
            state_widget.value = translate(_("{state} {date} by {principal}")).format(
                state=translate(workflow.get_state_label(state.state)),
                date=format_datetime(state.state_date),
                principal=get_principal(self.request, state.state_principal).title)
        state_urgency = self.widgets.get('state_urgency')
        if state_urgency is not None:
            converter = IDataConverter(state_urgency)
            state_urgency.value = converter.to_widget_value(state.state_urgency)
        state_comment = self.widgets.get('comment')
        if state_comment is not None:
            history_item = state.history[-1]
            state_comment.label = _("Associated comment")
            state_comment.value = history_item.comment
        self.parent_form.widgets.ignore_context = False


@adapter_config(name='publication',
                required=(IWfSharedContent, IAdminLayer, SharedContentSummaryView),
                provides=IGroup)
class SharedContentPublicationInfo(Group):
    """Shared content version info"""

    def __new__(cls, context, request, form):
        info = IWorkflowPublicationInfo(context, None)
        if (info is None) or not info.publication_effective_date:
            return None
        return Group.__new__(cls)

    legend = _("Publication dates")
    weight = 30

    fields = Fields(IWorkflowPublicationInfo).select('publication_effective_date',
                                                     'push_end_date',
                                                     'publication_expiration_date',
                                                     'displayed_publication_date')
    mode = DISPLAY_MODE


@adapter_config(name='version',
                required=(IWfSharedContent, IAdminLayer, SharedContentSummaryView),
                provides=IGroup)
class SharedContentVersionInfo(Group):
    """Shared content version info"""

    def __new__(cls, context, request, form):
        state = IWorkflowState(context, None)
        if state is None:
            return None
        return Group.__new__(cls)

    legend = _("Current version")
    weight = 40

    fields = Fields(IWorkflowState).select('version_id', 'state') + \
        Fields(IWfSharedContent).select('creation_label') + \
        Fields(IWfSharedContentRoles).select('owner') + \
        Fields(IWfSharedContent).select('last_update_label', 'modifiers')
    mode = DISPLAY_MODE

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        version_id = self.widgets.get('version_id')
        if version_id is not None:
            version_id.label = _("Version")
        state_widget = self.widgets.get('state')
        if state_widget is not None:
            translate = self.request.localizer.translate
            workflow = IWorkflow(self.context)
            workflow_state = IWorkflowState(self.context)
            history_item = workflow_state.history[-1]
            state_widget.value = translate(_("{state} since {date}, by {principal}")).format(
                state=translate(workflow.get_state_label(workflow_state.state)),
                date=format_datetime(tztime(history_item.date)),
                principal=get_principal(self.request, history_item.principal).title)


@adapter_config(name='history',
                required=(IWfSharedContent, IAdminLayer, SharedContentSummaryView),
                provides=IGroup)
class SharedContentHistoryInfo(Group):
    """Shared content history info"""

    def __new__(cls, context, request, form):
        info = IWorkflowPublicationInfo(context, None)
        if info is None:
            return None
        return Group.__new__(cls)

    legend = _("Content history")
    weight = 50

    @property
    def fields(self):
        fields = Fields(IWorkflowPublicationInfo).select('first_publication_date')
        state = IWorkflowState(self.context)
        if state.version_id > 1:
            fields += Fields(IWorkflowPublicationInfo).select('content_publication_date')
        fields += Fields(IWfSharedContent).select('first_owner')
        return fields

    mode = DISPLAY_MODE

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        info = IWorkflowPublicationInfo(self.context, None)
        version_date = self.widgets.get('first_publication_date')
        if version_date is not None:
            version_date.value = format_datetime(info.first_publication_date)
        content_date = self.widgets.get('content_publication_date')
        if content_date is not None:
            content_date.value = format_datetime(info.content_publication_date)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.zmi.thesaurus module

This module provides management components for thesaurus term
illustration extension.
"""

from pyams_content.component.illustration import IBaseIllustration, IIllustration
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_thesaurus.interfaces.term import IThesaurusTerm
from pyams_thesaurus.zmi.extension import ThesaurusTermExtensionEditForm
from pyams_utils.adapter import adapter_config


__docformat__ = 'restructuredtext'

from pyams_content import _


@ajax_form_config(name='illustration-dialog.html',
                  context=IThesaurusTerm, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ThesaurusTermIllustrationPropertiesEditForm(ThesaurusTermExtensionEditForm):
    """Thesaurus term illustration properties edit form"""

    subtitle = _("Associated illustration")
    legend = _("Illustration properties")
    modal_class = 'modal-xl'

    fields = Fields(IBaseIllustration)


@adapter_config(required=(IThesaurusTerm, IPyAMSLayer, ThesaurusTermIllustrationPropertiesEditForm),
                provides=IFormContent)
def thesaurus_term_illustration_form_content(context, request, form):
    """Thesaurus term illustration edit form content getter"""
    return IIllustration(context)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.portlet.header module

This module defines a shared content header portlet.
"""

from zope.schema.fieldproperty import FieldProperty

from pyams_content.shared.common.portlet.interfaces import HEADER_PORTLET_NAME, ISharedContentHeaderPortletSettings
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(provided=ISharedContentHeaderPortletSettings)
class SharedContentHeaderPortletSettings(PortletSettings):
    """Shared content portlet settings"""

    display_illustration = FieldProperty(ISharedContentHeaderPortletSettings['display_illustration'])
    display_breadcrumbs = FieldProperty(ISharedContentHeaderPortletSettings['display_breadcrumbs'])
    display_title = FieldProperty(ISharedContentHeaderPortletSettings['display_title'])
    display_tags = FieldProperty(ISharedContentHeaderPortletSettings['display_tags'])
    display_header = FieldProperty(ISharedContentHeaderPortletSettings['display_header'])
    display_publication_date = FieldProperty(ISharedContentHeaderPortletSettings['display_publication_date'])
    publication_date_prefix = FieldProperty(ISharedContentHeaderPortletSettings['publication_date_prefix'])
    display_alerts = FieldProperty(ISharedContentHeaderPortletSettings['display_alerts'])
    display_specificities = FieldProperty(ISharedContentHeaderPortletSettings['display_specificities'])


@portlet_config(permission=None)
class SharedContentHeaderPortlet(Portlet):
    """Shared content header portlet"""

    name = HEADER_PORTLET_NAME
    label = _("Content header")

    settings_factory = ISharedContentHeaderPortletSettings
    toolbar_css_class = 'fas fa-header'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.workflow.task module

This module defines scheduler tasks which are used to automatically publish or retire
contents which were published with a future publication date of with a planned retiring date.
"""

import logging
from datetime import datetime, timedelta, timezone

from pyramid.events import subscriber
from transaction.interfaces import ITransactionManager
from zope.interface import implementer
from zope.intid import IIntIds

from pyams_content.workflow.interfaces import IWorkflowManagementTask
from pyams_scheduler.interfaces import ISchedulerProcess, SCHEDULER_NAME
from pyams_scheduler.interfaces.task import IDateTaskScheduling, TASK_STATUS_OK
from pyams_scheduler.process import TaskResettingThread
from pyams_scheduler.task import Task
from pyams_security.interfaces.names import INTERNAL_USER_ID
from pyams_site.interfaces import PYAMS_APPLICATION_SETTINGS_KEY, PYAMS_APPLICATION_DEFAULT_NAME
from pyams_utils.registry import get_pyramid_registry, get_utility
from pyams_utils.timezone import gmtime
from pyams_utils.zodb import ZODBConnection
from pyams_workflow.interfaces import IWorkflow, IWorkflowInfo, IWorkflowState
from pyams_zmq.interfaces import IZMQProcessStartedEvent

__docformat__ = 'restructuredtext'

from pyams_content import _


LOGGER = logging.getLogger('PyAMS (content)')


@implementer(IWorkflowManagementTask)
class ContentPublishingTask(Task):
    """Content publisher task"""

    label = _("Content publisher task")
    icon_class = 'fas fa-share'

    settings_view_name = None
    principal_id = INTERNAL_USER_ID

    def __init__(self, oid, transition_id):
        super().__init__()
        self.oid = oid
        self.transition_id = transition_id

    def run(self, report, **kwargs):
        intids = get_utility(IIntIds)
        content = intids.queryObject(self.oid)
        if content is None:
            LOGGER.debug(">>> can't find publisher task target with OID {}".format(self.oid))
        else:
            workflow = IWorkflow(content)
            state = IWorkflowState(content)
            if state.state in workflow.visible_states:
                LOGGER.debug(">>> content is already published!")
            else:
                info = IWorkflowInfo(content)
                info.fire_transition(self.transition_id,
                                     check_security=False,
                                     principal=self.principal_id)
                info.fire_automatic()
        # remove task after execution!
        if self.__parent__ is not None:
            del self.__parent__[self.__name__]
        return TASK_STATUS_OK, None


@implementer(IWorkflowManagementTask)
class ContentArchivingTask(Task):
    """Content archiving task"""

    label = _("Content archiver task")
    icon_class = 'fas fa-reply'

    settings_view_name = None
    principal_id = INTERNAL_USER_ID

    def __init__(self, oid):
        super().__init__()
        self.oid = oid

    def run(self, report, **kwargs):
        intids = get_utility(IIntIds)
        content = intids.queryObject(self.oid)
        if content is None:
            LOGGER.debug(">>> can't find archiving task target with OID {}".format(self.oid))
        else:
            workflow = IWorkflow(content)
            state = IWorkflowState(content)
            if state.state not in workflow.visible_states:
                LOGGER.debug(">>> content is not currently published!")
            else:
                info = IWorkflowInfo(content)
                info.fire_transition_toward(workflow.auto_retired_state,
                                            check_security=False,
                                            principal=self.principal_id)
                info.fire_automatic()
        # remove task after execution!
        if self.__parent__ is not None:
            del self.__parent__[self.__name__]
        return TASK_STATUS_OK, None


@subscriber(IZMQProcessStartedEvent, context_selector=ISchedulerProcess)
def handle_scheduler_start(event):
    """Check for scheduler tasks

    Workflow management tasks are typically automatically deleted after their execution.
    If tasks with passed execution date are still present in the scheduler, this is generally
    because scheduler was stopped at task execution time; so tasks which where not run are
    re-scheduled at process startup in a very near future...
    """
    with ZODBConnection() as root:
        registry = get_pyramid_registry()
        application_name = registry.settings.get(PYAMS_APPLICATION_SETTINGS_KEY,
                                                 PYAMS_APPLICATION_DEFAULT_NAME)
        application = root.get(application_name)
        sm = application.getSiteManager()  # pylint: disable=invalid-name
        scheduler = sm.get(SCHEDULER_NAME)
        if scheduler is not None:
            LOGGER.debug("Checking pending scheduler tasks on {!r}".format(scheduler))
            for task in scheduler.values():
                if not IWorkflowManagementTask.providedBy(task):
                    continue
                schedule_info = IDateTaskScheduling(task, None)
                if schedule_info is None:  # no date scheduling
                    continue
                now = datetime.now(timezone.utc)
                if schedule_info.active and (schedule_info.start_date < now):
                    # we add a small amount of time to be sure that scheduler and indexer
                    # processes are started...
                    schedule_info.start_date = now + timedelta(minutes=1)
                    # commit update for reset thread to get updated data!!
                    ITransactionManager(task).commit()
                    # start task resetting thread
                    LOGGER.debug(" - restarting task « {} »".format(task.name))
                    TaskResettingThread(event.object, task).start()



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.folder module

This module provides persistent classes to handles sites folders, which are used to group
contents together.
"""

from pyramid.events import subscriber
from zope.container.ordered import OrderedContainer
from zope.interface import alsoProvides, implementer
from zope.intid import IIntIds
from zope.lifecycleevent import IObjectAddedEvent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.component.illustration.interfaces import IIllustrationTarget, \
    ILinkIllustrationTarget
from pyams_content.component.links import InternalLink
from pyams_content.component.thesaurus import IThemesTarget
from pyams_content.feature.navigation.interfaces import IDynamicMenu
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.shared.common import ISharedContent
from pyams_content.shared.common.manager import BaseSharedTool
from pyams_content.shared.site.container import SiteContainerMixin
from pyams_content.shared.site.interfaces import ISiteFolder, ISiteManager, \
    SITE_FOLDERS_VOCABULARY, SITE_TOPIC_CONTENT_TYPE
from pyams_i18n.interfaces import II18n
from pyams_portal.interfaces import IPortalContext, IPortalFooterContext, IPortalHeaderContext
from pyams_security.interfaces import IDefaultProtectionPolicy, IViewContextPermissionChecker
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config, get_object_factory
from pyams_utils.finder import find_objects_providing
from pyams_utils.registry import get_utility
from pyams_utils.request import query_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(ISiteFolder)
@implementer(IDefaultProtectionPolicy,
             IIllustrationTarget, ILinkIllustrationTarget,
             IPortalContext, IPortalHeaderContext, IPortalFooterContext, IPreviewTarget)
class SiteFolder(SiteContainerMixin, OrderedContainer, BaseSharedTool):
    """Site folder persistent class"""

    header = FieldProperty(ISiteFolder['header'])
    description = FieldProperty(ISiteFolder['description'])
    notepad = FieldProperty(ISiteFolder['notepad'])

    visible_in_list = FieldProperty(ISiteFolder['visible_in_list'])
    navigation_title = FieldProperty(ISiteFolder['navigation_title'])
    navigation_mode = FieldProperty(ISiteFolder['navigation_mode'])

    content_name = _("Site folder")

    sequence_name = ''  # use default sequence generator
    sequence_prefix = ''

    shared_content_type = SITE_TOPIC_CONTENT_TYPE

    @property
    def shared_content_factory(self):
        return get_object_factory(ISharedContent, name=self.shared_content_type)

    def is_deletable(self):
        """Check if item can be deleted"""
        for element in self.values():
            if not element.is_deletable():
                return False
        return True


@subscriber(IObjectAddedEvent, context_selector=ISiteFolder, parent_selector=IThemesTarget)
def handle_added_site_folder(event):
    """Handle site folder when added to a themes target"""
    alsoProvides(event.object, IThemesTarget)


@adapter_config(required=ISiteFolder,
                provides=IViewContextPermissionChecker)
class SiteFolderPermissionChecker(ContextAdapter):
    """Site folder edit permission checker"""

    edit_permission = MANAGE_SITE_PERMISSION


@adapter_config(required=ISiteFolder,
                provides=IDynamicMenu)
def site_folder_dynamic_menu(context):
    """Site folder dynamic menu factory"""
    result = InternalLink()
    result.title = context.navigation_title.copy() if context.navigation_title else {}
    result.reference = ISequentialIdInfo(context).hex_oid
    return result


@vocabulary_config(name=SITE_FOLDERS_VOCABULARY)
class SiteManagerFoldersVocabulary(SimpleVocabulary):
    """Site manager folders vocabulary"""

    def __init__(self, context):
        terms = []
        site = get_parent(context, ISiteManager)
        if site is not None:
            request = query_request()
            intids = get_utility(IIntIds)
            for folder in find_objects_providing(site, ISiteFolder):
                terms.append(SimpleTerm(value=intids.queryId(folder),
                                        title=II18n(folder).query_attribute('title',
                                                                            request=request)))
        super().__init__(terms)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.skin.illustration module

This module provides base illustrations renderers.
"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration import IParagraphIllustration
from pyams_content.component.illustration.skin.interfaces import \
    IIllustrationDefaultRendererSettings, IIllustrationRenderer, ILLUSTRATION_AFTER_BODY, \
    ILLUSTRATION_BEFORE_BODY, IParagraphIllustrationSideRendererSettings
from pyams_content.feature.renderer import BaseContentRenderer, IContentRenderer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Illustrations renderer settings
#

@factory_config(IIllustrationDefaultRendererSettings)
class IllustrationDefaultRendererSettings(Persistent, Contained):
    """Illustration paragraph default renderer settings"""

    thumb_selection = FieldProperty(
        IIllustrationDefaultRendererSettings['thumb_selection'])


@factory_config(IParagraphIllustrationSideRendererSettings)
class ParagraphIllustrationSideRendererSettings(IllustrationDefaultRendererSettings):
    """Illustration paragraph side renderer settings"""

    thumb_selection = FieldProperty(
        IParagraphIllustrationSideRendererSettings['thumb_selection'])
    zoom_on_click = FieldProperty(
        IParagraphIllustrationSideRendererSettings['zoom_on_click'])


#
# Illustrations renderers
#

@implementer(IIllustrationRenderer)
class BaseIllustrationRenderer(BaseContentRenderer):
    """Base illustration renderer"""

    settings_interface = IIllustrationDefaultRendererSettings
    position = None


@adapter_config(name='centered-before',
                required=(IParagraphIllustration, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class ParagraphIllustrationBeforeTextRenderer(BaseIllustrationRenderer):
    """Illustration centered before text renderer"""

    label = _("Centered image before text")

    position = ILLUSTRATION_BEFORE_BODY
    weight = 10


@template_config(template='templates/illustration-side.pt', layer=IPyAMSLayer)
class BaseParagraphIllustrationSideRenderer(BaseIllustrationRenderer):
    """Base illustration side renderer"""

    settings_interface = IParagraphIllustrationSideRendererSettings
    position = ILLUSTRATION_BEFORE_BODY

    css_class = None


@adapter_config(name='float-left',
                required=(IParagraphIllustration, IPyAMSLayer),
                provides=IContentRenderer)
class ParagraphIllustrationLeftFloatRenderer(BaseParagraphIllustrationSideRenderer):
    """Left floating illustration renderer"""

    label = _("Floating illustration to the left")
    css_class = 'float-left'

    weight = 20


@adapter_config(name='float-right',
                required=(IParagraphIllustration, IPyAMSLayer),
                provides=IContentRenderer)
class ParagraphIllustrationRightFloatRenderer(BaseParagraphIllustrationSideRenderer):
    """Right floating illustration renderer"""

    label = _("Floating illustration to the right")
    css_class = 'float-right'

    weight = 30


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IParagraphIllustration, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class ParagraphIllustrationAfterTextRenderer(BaseIllustrationRenderer):
    """Illustration centered after text renderer"""

    label = _("Centered image after text (default)")

    position = ILLUSTRATION_AFTER_BODY
    weight = 40



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.file module

"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.interface import alsoProvides
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent, \
    ObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.gallery import IBaseGallery, IGalleryFile
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_content.shared.common import IWfSharedContent
from pyams_file.interfaces import IImageFile, IResponsiveImage
from pyams_file.property import FileProperty
from pyams_i18n.interfaces import INegotiator
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_current_registry, get_utility
from pyams_utils.traversing import get_parent
from pyams_workflow.content import HiddenContentPublicationInfo
from pyams_workflow.interfaces import IWorkflowPublicationInfo


#
# Gallery file
#

@factory_config(IGalleryFile)
class GalleryFile(Persistent, Contained):
    """Gallery file info"""

    _data = FileProperty(IGalleryFile['data'])
    _title = FieldProperty(IGalleryFile['title'])
    alt_title = FieldProperty(IGalleryFile['alt_title'])
    description = FieldProperty(IGalleryFile['description'])
    author = FieldProperty(IGalleryFile['author'])
    sound = FileProperty(IGalleryFile['sound'])
    sound_title = FieldProperty(IGalleryFile['sound_title'])
    sound_description = FieldProperty(IGalleryFile['sound_description'])
    visible = FieldProperty(IGalleryFile['visible'])

    @property
    def data(self):
        """File data getter"""
        return self._data

    @data.setter
    def data(self, value):
        """File data setter"""
        self._data = value
        if IImageFile.providedBy(self._data):
            alsoProvides(self._data, IResponsiveImage)

    @data.deleter
    def data(self):
        """File data deleter"""
        del self._data

    @property
    def title(self):
        """File title getter"""
        return self._title

    @title.setter
    def title(self, value):
        """File title setter"""
        self._title = value
        if self._data:
            negociator = get_utility(INegotiator)
            self._data.title = value.get(negociator.server_language)


@adapter_config(required=IGalleryFile,
                provides=IViewContextPermissionChecker)
class GalleryFilePermissionChecker(ContextAdapter):
    """Gallery file permission checker"""

    @property
    def edit_permission(self):
        """File edit permission getter"""
        gallery = get_parent(self.context, IBaseGallery)
        if gallery is not None:
            return IViewContextPermissionChecker(gallery).edit_permission
        return MANAGE_SITE_ROOT_PERMISSION


@subscriber(IObjectAddedEvent, context_selector=IGalleryFile)
def handle_added_gallery_file(event):
    """Handle added gallery file"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IGalleryFile)
def handle_modified_gallery_file(event):
    """Handle modified gallery file"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IGalleryFile)
def handle_removed_gallery_file(event):
    """Handle removed gallery file"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@adapter_config(required=IGalleryFile,
                provides=IWorkflowPublicationInfo)
def gallery_file_publication_info(context):
    """Gallery file publication info"""
    if not context.visible:
        return HiddenContentPublicationInfo()
    return None



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.thesaurus module

This module provides persistent classes and adapters used to handle views
thesaurus-based settings support and catalog-based queries.
"""

__docformat__ = 'restructuredtext'

from hypatia.interfaces import ICatalog
from hypatia.query import Eq, Any
from persistent import Persistent
from zope.container.contained import Contained
from zope.intid.interfaces import IIntIds
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.thesaurus import ICollectionsInfo, ITagsInfo, IThemesInfo
from pyams_content.shared.view import IViewSettings, IWfView
from pyams_content.shared.view.interfaces.query import IViewQueryParamsExtension
from pyams_content.shared.view.interfaces.settings import IViewCollectionsSettings, IViewTagsSettings, \
    IViewThemesSettings, VIEW_COLLECTIONS_SETTINGS_KEY, VIEW_TAGS_SETTINGS_KEY, VIEW_THEMES_SETTINGS_KEY
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility


#
# Tags management
#

@factory_config(IViewTagsSettings)
class ViewTagsSettings(Persistent, Contained):
    """View tags settings"""

    select_context_tags = FieldProperty(IViewTagsSettings['select_context_tags'])
    tags = FieldProperty(IViewTagsSettings['tags'])

    @property
    def is_using_context(self):
        """Context usage checker"""
        return self.select_context_tags

    def get_tags(self, context):
        """Selection tags getter"""
        tags = set()
        if self.select_context_tags:
            tags_info = ITagsInfo(context, None)
            if tags_info is not None:
                tags |= set(tags_info.tags or ())
        if self.tags:
            tags |= set(self.tags)
        return tags

    def get_tags_index(self, context):
        """Tags index values getter"""
        intids = get_utility(IIntIds)
        return [
            intids.register(term)
            for term in self.get_tags(context)
        ]


@adapter_config(required=IWfView,
                provides=IViewTagsSettings)
@adapter_config(name='tags',
                required=IWfView,
                provides=IViewSettings)
def view_tags_settings(view):
    """View tags settings factory"""
    return get_annotation_adapter(view, VIEW_TAGS_SETTINGS_KEY,
                                  IViewTagsSettings,
                                  name='++view:tags++')


@adapter_config(name='tags',
                required=IWfView,
                provides=IViewQueryParamsExtension)
class ViewTagsQueryParamsExtension(ContextAdapter):
    """View tags query params extension"""

    weight = 50

    def get_params(self, context, request=None):
        """Query params getter"""
        catalog = get_utility(ICatalog)
        settings = IViewTagsSettings(self.context)
        # check tags
        tags = settings.get_tags_index(context)
        if tags:
            yield Any(catalog['tags'], tags)
        elif settings.select_context_tags:
            yield None


#
# Themes management
#

@factory_config(IViewThemesSettings)
class ViewThemesSettings(Persistent, Contained):
    """View themes settings"""

    select_context_themes = FieldProperty(IViewThemesSettings['select_context_themes'])
    themes = FieldProperty(IViewThemesSettings['themes'])
    include_subthemes = FieldProperty(IViewThemesSettings['include_subthemes'])

    @property
    def is_using_context(self):
        return self.select_context_themes

    def get_themes(self, context):
        themes = set()
        if self.select_context_themes:
            themes_info = IThemesInfo(context, None)
            if themes_info is not None:
                themes |= set(themes_info.themes or ())
        if self.themes:
            themes |= set(self.themes)
        if self.include_subthemes:
            for theme in themes.copy():
                themes |= set(theme.get_all_childs())
        return themes

    def get_themes_index(self, context):
        intids = get_utility(IIntIds)
        return [intids.register(term) for term in self.get_themes(context)]


@adapter_config(context=IWfView,
                provides=IViewThemesSettings)
@adapter_config(name='themes',
                context=IWfView,
                provides=IViewSettings)
def view_themes_settings(view):
    """View themes settings factory"""
    return get_annotation_adapter(view, VIEW_THEMES_SETTINGS_KEY,
                                  IViewThemesSettings,
                                  name='++view:themes++')


@adapter_config(name='themes',
                context=IWfView,
                provides=IViewQueryParamsExtension)
class ViewThemesQueryParamsExtension(ContextAdapter):
    """View themes query params extension"""

    weight = 52

    def get_params(self, context, request=None):
        """Query params getter"""
        catalog = get_utility(ICatalog)
        settings = IViewThemesSettings(self.context)
        # check themes
        themes = settings.get_themes_index(context)
        if themes:
            yield Any(catalog['themes'], themes)
        elif settings.select_context_themes:
            yield None


#
# Collections management
#

@factory_config(IViewCollectionsSettings)
class ViewCollectionsSettings(Persistent, Contained):
    """View collections settings"""

    select_context_collections = FieldProperty(IViewCollectionsSettings['select_context_collections'])
    collections = FieldProperty(IViewCollectionsSettings['collections'])

    @property
    def is_using_context(self):
        return self.select_context_collections

    def get_collections(self, context):
        collections = set()
        if self.select_context_collections:
            collections_info = ICollectionsInfo(context, None)
            if collections_info is not None:
                collections |= set(collections_info.collections or ())
        if self.collections:
            collections |= set(self.collections)
        return collections

    def get_collections_index(self, context):
        intids = get_utility(IIntIds)
        return [intids.register(term) for term in self.get_collections(context)]


@adapter_config(required=IWfView,
                provides=IViewCollectionsSettings)
@adapter_config(name='collections',
                required=IWfView,
                provides=IViewSettings)
def view_collections_settings(view):
    """View collections settings factory"""
    return get_annotation_adapter(view, VIEW_COLLECTIONS_SETTINGS_KEY,
                                  IViewCollectionsSettings,
                                  name='++view:collections++')


@adapter_config(name='collections',
                context=IWfView,
                provides=IViewQueryParamsExtension)
class ViewCollectionsQueryParamsExtension(ContextAdapter):
    """View collections query params extension"""

    weight = 54

    def get_params(self, context, request=None):
        """Query params getter"""
        catalog = get_utility(ICatalog)
        settings = IViewCollectionsSettings(self.context)
        # check collections
        collections = settings.get_collections_index(context)
        if collections:
            yield Any(catalog['collections'], collections)
        elif settings.select_context_collections:
            yield None



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.root.interfaces module

This module defines interfaces which are used for whole site configuration.
"""

from zope.interface import Interface
from zope.schema import Dict, InterfaceField, TextLine, URI

from pyams_content.interfaces import OPERATOR_ROLE, WEBMASTER_ROLE
from pyams_file.schema import ImageField
from pyams_i18n.schema import I18nTextField, I18nTextLineField
from pyams_portal.interfaces import DESIGNER_ROLE
from pyams_security.interfaces import IContentRoles
from pyams_security.schema import PrincipalField, PrincipalsSetField


__docformat__ = 'restructuredtext'

from pyams_content import _
from pyams_utils.schema import MailAddressField

SITE_ROOT_INFOS_KEY = 'pyams_content.root'


class ISiteRootInfos(Interface):
    """Site root information interfaces"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Application title displayed in title bar"),
                              required=False)

    short_title = I18nTextLineField(title=_("Short title"),
                                    description=_("Application short title visible as "
                                                  "title prefix"),
                                    required=False)

    description = I18nTextField(title=_("Description"),
                                description=_("Main application description"),
                                required=False)

    author = TextLine(title=_("Author"),
                      description=_("Public author name"),
                      required=False)

    icon = ImageField(title=_("Icon"),
                      description=_("Browser favourite icon"),
                      required=False)

    logo = ImageField(title=_("Logo"),
                      description=_("Image containing application logo"),
                      required=False)

    public_url = URI(title=_("Public site URI"),
                     description=_("Base URL of the public site"),
                     required=False)

    support_email = MailAddressField(title=_("Support email"),
                                     description=_("Public support email address"),
                                     required=False)


SITEROOT_ROLES = 'pyams_content.root.roles'


class ISiteRootRoles(IContentRoles):
    """Main site roles"""

    webmasters = PrincipalsSetField(title=_("Webmasters"),
                                    description=_("These principals can handle all settings and "
                                                  "contents of web site, except for a few ones "
                                                  "which require administrator access"),
                                    role_id=WEBMASTER_ROLE,
                                    required=False)

    designers = PrincipalsSetField(title=_("Templates managers"),
                                   description=_("These principals can only handle presentation "
                                                 "templates"),
                                   role_id=DESIGNER_ROLE,
                                   required=False)

    operators = PrincipalField(title=_("Operators group"),
                               description=_("Name of group containing all roles owners"),
                               role_id=OPERATOR_ROLE,
                               required=False)

    def get_operators_group(self):
        """Get operators groups instance"""


SITE_ROOT_TOOLS_CONFIGURATION_KEY = 'pyams_content.root.configuration'


class ISiteRootToolsConfiguration(Interface):
    """Site root shared tools configuration"""

    tables_manager_name = TextLine(title="Tables manager name")
    tables_names = Dict(title="References tables names",
                        key_type=InterfaceField(),
                        value_type=TextLine())

    def check_table(self, interface, table_name, registry):
        """Check or create given table from factory

        :param interface: provided table interface
        :param table_name: default table name
        :param registry: current registry
        """

    tools_manager_name = TextLine(title="Shared tools manager name")
    tools_names = Dict(title="Shared tools names",
                       key_type=InterfaceField(),
                       value_type=TextLine())

    def check_tool(self, interface, tool_name, registry):
        """Check or create given shared tool from factory

        :param interface: provided shared tool interface
        :param tool_name: default tool name
        :param registry: current registry
        """
